@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2009 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2014 by Paolo Lucente
 */
 
 /*
@@ -27,18 +27,29 @@
 #include "crc32.c"
 
 /* functions */
-struct acc *search_accounting_structure(struct pkt_primitives *addr, struct pkt_bgp_primitives *pbgp)
+struct acc *search_accounting_structure(struct primitives_ptrs *prim_ptrs)
 {
+  struct pkt_data *data = prim_ptrs->data;
+  struct pkt_primitives *addr = &data->primitives;
+  struct pkt_bgp_primitives *pbgp = prim_ptrs->pbgp;
+  struct pkt_nat_primitives *pnat = prim_ptrs->pnat;
+  struct pkt_mpls_primitives *pmpls = prim_ptrs->pmpls;
+  char *pcust = prim_ptrs->pcust;
+  struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   struct acc *elem_acc;
   unsigned int hash, pos;
   unsigned int pp_size = sizeof(struct pkt_primitives); 
   unsigned int pb_size = sizeof(struct pkt_bgp_primitives);
+  unsigned int pn_size = sizeof(struct pkt_nat_primitives);
+  unsigned int pm_size = sizeof(struct pkt_mpls_primitives);
+  unsigned int pc_size = config.cpptrs.len;
 
   hash = cache_crc32((unsigned char *)addr, pp_size);
-  /* XXX: to be optimized? */
-  if (PbgpSz) {
-    if (pbgp) hash ^= cache_crc32((unsigned char *)pbgp, pb_size);
-  }
+  if (pbgp) hash ^= cache_crc32((unsigned char *)pbgp, pb_size);
+  if (pnat) hash ^= cache_crc32((unsigned char *)pnat, pn_size);
+  if (pmpls) hash ^= cache_crc32((unsigned char *)pmpls, pm_size);
+  if (pcust && pc_size) hash ^= cache_crc32((unsigned char *)pcust, pc_size);
+  if (pvlen) hash ^= cache_crc32((unsigned char *)pvlen, (PvhdrSz + pvlen->tot_len));
   pos = hash % config.buckets;
 
   Log(LOG_DEBUG, "DEBUG ( %s/%s ): Selecting bucket %u.\n", config.name, config.type, pos);
@@ -48,8 +59,7 @@ struct acc *search_accounting_structure(struct pkt_primitives *addr, struct pkt_
   
   while (elem_acc) {
     if (elem_acc->signature == hash) {
-      if (compare_accounting_structure(elem_acc, addr, pbgp) == 0) return elem_acc;
-      // if (!memcmp(&elem_acc->primitives, addr, sizeof(struct pkt_primitives))) return elem_acc;
+      if (compare_accounting_structure(elem_acc, prim_ptrs) == 0) return elem_acc;
     }
     elem_acc = elem_acc->next;
   } 
@@ -57,35 +67,63 @@ struct acc *search_accounting_structure(struct pkt_primitives *addr, struct pkt_
   return NULL;
 }
 
-int compare_accounting_structure(struct acc *elem, struct pkt_primitives *data, struct pkt_bgp_primitives *pbgp)
+int compare_accounting_structure(struct acc *elem, struct primitives_ptrs *prim_ptrs)
 {
-  int res_data = TRUE, res_bgp = TRUE; 
+  struct pkt_data *pdata = prim_ptrs->data;
+  struct pkt_primitives *data = &pdata->primitives;
+  struct pkt_bgp_primitives *pbgp = prim_ptrs->pbgp;
+  struct pkt_nat_primitives *pnat = prim_ptrs->pnat;
+  struct pkt_mpls_primitives *pmpls = prim_ptrs->pmpls;
+  char *pcust = prim_ptrs->pcust;
+  struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
+  int res_data = TRUE, res_bgp = TRUE, res_nat = TRUE, res_mpls = TRUE, res_cust = TRUE; 
+  int res_vlen = TRUE;
 
   res_data = memcmp(&elem->primitives, data, sizeof(struct pkt_primitives));
 
-  /* XXX: to be optimized? */
-  if (PbgpSz) {
+  if (pbgp) {
     if (elem->cbgp) {
       struct pkt_bgp_primitives tmp_pbgp;
 
-      cache_to_pkt_bgp_primitives(&tmp_pbgp, elem->cbgp);
+      cache_to_pkt_bgp_primitives(&tmp_pbgp, elem->cbgp, config.what_to_count);
       res_bgp = memcmp(&tmp_pbgp, pbgp, sizeof(struct pkt_bgp_primitives));
     }
   }
   else res_bgp = FALSE;
 
-  return res_data | res_bgp;
+  if (pnat && elem->pnat) res_nat = memcmp(elem->pnat, pnat, sizeof(struct pkt_nat_primitives));
+  else res_nat = FALSE;
+
+  if (pmpls && elem->pmpls) res_mpls = memcmp(elem->pmpls, pmpls, sizeof(struct pkt_mpls_primitives));
+  else res_mpls = FALSE;
+
+  if (pcust && elem->pcust) res_cust = memcmp(elem->pcust, pcust, config.cpptrs.len);
+  else res_cust = FALSE;
+
+  if (pvlen && elem->pvlen) res_vlen = vlen_prims_cmp(elem->pvlen, pvlen);
+  else res_vlen = FALSE;
+
+  return res_data | res_bgp | res_nat | res_mpls | res_cust | res_vlen;
 }
 
-void insert_accounting_structure(struct pkt_data *data, struct pkt_bgp_primitives *pbgp)
+void insert_accounting_structure(struct primitives_ptrs *prim_ptrs)
 {
+  struct pkt_data *data = prim_ptrs->data;
   struct pkt_primitives *addr = &data->primitives;
+  struct pkt_bgp_primitives *pbgp = prim_ptrs->pbgp;
+  struct pkt_nat_primitives *pnat = prim_ptrs->pnat;
+  struct pkt_mpls_primitives *pmpls = prim_ptrs->pmpls;
+  char *pcust = prim_ptrs->pcust;
+  struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   struct acc *elem_acc;
   unsigned char *elem, *new_elem;
   int solved = FALSE;
   unsigned int hash, pos;
   unsigned int pp_size = sizeof(struct pkt_primitives);
   unsigned int pb_size = sizeof(struct pkt_bgp_primitives);
+  unsigned int pn_size = sizeof(struct pkt_nat_primitives);
+  unsigned int pm_size = sizeof(struct pkt_mpls_primitives);
+  unsigned int pc_size = config.cpptrs.len;
   unsigned int cb_size = sizeof(struct cache_bgp_primitives);
 
   /* We are classifing packets. We have a non-zero bytes accumulator (ba)
@@ -95,7 +133,7 @@ void insert_accounting_structure(struct pkt_data *data, struct pkt_bgp_primitive
     pm_class_t lclass = data->primitives.class;
 
     data->primitives.class = 0;
-    elem_acc = search_accounting_structure(&data->primitives, pbgp);
+    elem_acc = search_accounting_structure(prim_ptrs);
     data->primitives.class = lclass;
 
     /* We can assign the flow to a new class only if we are able to subtract
@@ -118,10 +156,11 @@ void insert_accounting_structure(struct pkt_data *data, struct pkt_bgp_primitive
   elem = a;
 
   hash = cache_crc32((unsigned char *)addr, pp_size);
-  /* XXX: to be optimized? */
-  if (PbgpSz) {
-    if (pbgp) hash ^= cache_crc32((unsigned char *)pbgp, pb_size);
-  }
+  if (pbgp) hash ^= cache_crc32((unsigned char *)pbgp, pb_size);
+  if (pnat) hash ^= cache_crc32((unsigned char *)pnat, pn_size);
+  if (pmpls) hash ^= cache_crc32((unsigned char *)pmpls, pm_size);
+  if (pcust && pc_size) hash ^= cache_crc32((unsigned char *)pcust, pc_size);
+  if (pvlen) hash ^= cache_crc32((unsigned char *)pvlen, (PvhdrSz + pvlen->tot_len));
   pos = hash % config.buckets;
       
   Log(LOG_DEBUG, "DEBUG ( %s/%s ): Selecting bucket %u.\n", config.name, config.type, pos);
@@ -132,13 +171,13 @@ void insert_accounting_structure(struct pkt_data *data, struct pkt_bgp_primitive
   if (lru_elem_ptr[pos]) {
     elem_acc = lru_elem_ptr[pos];
     if (elem_acc->signature == hash) {
-      if (compare_accounting_structure(elem_acc, addr, pbgp) == 0) { 
-      // if (memcmp(&elem_acc->primitives, addr, sizeof(struct pkt_primitives)) == 0) {
+      if (compare_accounting_structure(elem_acc, prim_ptrs) == 0) { 
         if (elem_acc->reset_flag) reset_counters(elem_acc);
         elem_acc->packet_counter += data->pkt_num;
         elem_acc->flow_counter += data->flo_num;
         elem_acc->bytes_counter += data->pkt_len;
 	elem_acc->tcp_flags |= data->tcp_flags;
+        elem_acc->flow_type = data->flow_type; 
         if (config.what_to_count & COUNT_CLASS) {
           elem_acc->packet_counter += data->cst.pa;
           elem_acc->bytes_counter += data->cst.ba;
@@ -154,13 +193,13 @@ void insert_accounting_structure(struct pkt_data *data, struct pkt_bgp_primitive
 
   while (solved == FALSE) {
     if (elem_acc->signature == hash) {
-      if (compare_accounting_structure(elem_acc, addr, pbgp) == 0) {
-      // if (memcmp(&elem_acc->primitives, addr, sizeof(struct pkt_primitives)) == 0) {
+      if (compare_accounting_structure(elem_acc, prim_ptrs) == 0) {
         if (elem_acc->reset_flag) reset_counters(elem_acc);
         elem_acc->packet_counter += data->pkt_num;
         elem_acc->flow_counter += data->flo_num;
         elem_acc->bytes_counter += data->pkt_len;
 	elem_acc->tcp_flags |= data->tcp_flags;
+        elem_acc->flow_type = data->flow_type;
 	if (config.what_to_count & COUNT_CLASS) {
 	  elem_acc->packet_counter += data->cst.pa;
 	  elem_acc->bytes_counter += data->cst.ba;
@@ -174,26 +213,86 @@ void insert_accounting_structure(struct pkt_data *data, struct pkt_bgp_primitive
       if (elem_acc->reset_flag) elem_acc->reset_flag = FALSE; 
       memcpy(&elem_acc->primitives, addr, sizeof(struct pkt_primitives));
 
-      /* XXX: to be optimized? */
-      if (PbgpSz) {
-	if (elem_acc->cbgp) {
-	  if (elem_acc->cbgp->std_comms) free(elem_acc->cbgp->std_comms);
-	  if (elem_acc->cbgp->ext_comms) free(elem_acc->cbgp->ext_comms);
-	  if (elem_acc->cbgp->as_path) free(elem_acc->cbgp->as_path);
-	  if (elem_acc->cbgp->src_std_comms) free(elem_acc->cbgp->src_std_comms);
-	  if (elem_acc->cbgp->src_ext_comms) free(elem_acc->cbgp->src_ext_comms);
-	  if (elem_acc->cbgp->src_as_path) free(elem_acc->cbgp->src_as_path);
-	  free(elem_acc->cbgp);
+      if (pbgp) {
+	if (!elem_acc->cbgp) {
+	  elem_acc->cbgp = (struct cache_bgp_primitives *) malloc(cb_size);
+	  if (!elem_acc->cbgp) {
+	    Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (insert_accounting_structure). Exiting ..\n", config.name, config.type);
+	    exit_plugin(1);
+          }
 	}
-	elem_acc->cbgp = (struct cache_bgp_primitives *) malloc(cb_size);
+
 	memset(elem_acc->cbgp, 0, cb_size);
         pkt_to_cache_bgp_primitives(elem_acc->cbgp, pbgp, config.what_to_count);
       }
+      else free_cache_bgp_primitives(&elem_acc->cbgp);
+
+      if (pnat) {
+	if (!elem_acc->pnat) {
+	  elem_acc->pnat = (struct pkt_nat_primitives *) malloc(pn_size);
+	  if (!elem_acc->pnat) {
+            Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (insert_accounting_structure). Exiting ..\n", config.name, config.type);
+            exit_plugin(1);
+	  }
+	}
+	memcpy(elem_acc->pnat, pnat, pn_size);
+      }
+      else {
+	if (elem_acc->pnat) free(elem_acc->pnat);
+	elem_acc->pnat = NULL;
+      }
+
+      if (pmpls) {
+	if (!elem_acc->pmpls) {
+	  elem_acc->pmpls = (struct pkt_mpls_primitives *) malloc(pm_size);
+	  if (!elem_acc->pmpls) {
+            Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (insert_accounting_structure). Exiting ..\n", config.name, config.type);
+            exit_plugin(1);
+	  }
+	}
+        memcpy(elem_acc->pmpls, pmpls, pm_size);
+      }
+      else {
+	if (elem_acc->pmpls) free(elem_acc->pmpls);
+	elem_acc->pmpls = NULL;
+      }
+
+      if (pcust) {
+	if (!elem_acc->pcust) {
+	  elem_acc->pcust = (char *) malloc(pc_size);
+	  if (!elem_acc->pcust) {
+            Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (insert_accounting_structure). Exiting ..\n", config.name, config.type);
+            exit_plugin(1);
+	  }
+	}
+        memcpy(elem_acc->pcust, pcust, pc_size);
+      }
+      else {
+	if (elem_acc->pcust) free(elem_acc->pcust);
+	elem_acc->pcust = NULL;
+      }
+
+      /* if we have a pvlen from before let's free it up due to the vlen nature of the memory area */
+      if (elem_acc->pvlen) {
+        vlen_prims_free(elem_acc->pvlen);
+        elem_acc->pvlen = NULL;
+      }
+
+      if (pvlen) {
+        if (!elem_acc->pvlen) {
+          elem_acc->pvlen = (struct pkt_vlen_hdr_primitives *) vlen_prims_copy(pvlen);
+          if (!elem_acc->pvlen) {
+            Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (insert_accounting_structure). Exiting ..\n", config.name, config.type);
+            exit_plugin(1);
+          }
+        }
+      }
 
       elem_acc->packet_counter += data->pkt_num;
       elem_acc->flow_counter += data->flo_num;
       elem_acc->bytes_counter += data->pkt_len;
       elem_acc->tcp_flags |= data->tcp_flags;
+      elem_acc->flow_type = data->flow_type;
       elem_acc->signature = hash;
       if (config.what_to_count & COUNT_CLASS) {
         elem_acc->packet_counter += data->cst.pa;
@@ -241,18 +340,68 @@ void insert_accounting_structure(struct pkt_data *data, struct pkt_bgp_primitive
       elem_acc = (struct acc *) new_elem;
       memcpy(&elem_acc->primitives, addr, sizeof(struct pkt_primitives));
 
-      /* XXX: to be optimized? */
-      if (PbgpSz) {
+      if (pbgp) {
         elem_acc->cbgp = (struct cache_bgp_primitives *) malloc(cb_size);
+        if (!elem_acc->cbgp) {
+          Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (insert_accounting_structure). Exiting ..\n", config.name, config.type);
+          exit_plugin(1);
+	}
         memset(elem_acc->cbgp, 0, cb_size);
         pkt_to_cache_bgp_primitives(elem_acc->cbgp, pbgp, config.what_to_count);
       }
       else elem_acc->cbgp = NULL;
 
+      if (pnat) {
+        elem_acc->pnat = (struct pkt_nat_primitives *) malloc(pn_size);
+	if (!elem_acc->pnat) {
+          Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (insert_accounting_structure). Exiting ..\n", config.name, config.type);
+          exit_plugin(1);
+	}
+        memcpy(elem_acc->pnat, pnat, pn_size);
+      }
+      else elem_acc->pnat = NULL;
+
+      if (pmpls) {
+        elem_acc->pmpls = (struct pkt_mpls_primitives *) malloc(pm_size);
+	if (!elem_acc->pmpls) {
+          Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (insert_accounting_structure). Exiting ..\n", config.name, config.type);
+          exit_plugin(1);
+	}
+        memcpy(elem_acc->pmpls, pmpls, pm_size);
+      }
+      else elem_acc->pmpls = NULL;
+
+      if (pcust) {
+        elem_acc->pcust = (char *) malloc(pc_size);
+	if (!elem_acc->pcust) {
+          Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (insert_accounting_structure). Exiting ..\n", config.name, config.type);
+          exit_plugin(1);
+	}
+        memcpy(elem_acc->pcust, pcust, pc_size);
+      }
+      else elem_acc->pcust = NULL;
+
+      /* if we have a pvlen from before let's free it up due to the vlen nature of the memory area */
+      if (elem_acc->pvlen) {
+        vlen_prims_free(elem_acc->pvlen);
+        elem_acc->pvlen = NULL;
+      }
+
+      if (pvlen) {
+        if (!elem_acc->pvlen) {
+          elem_acc->pvlen = (struct pkt_vlen_hdr_primitives *) vlen_prims_copy(pvlen);
+          if (!elem_acc->pvlen) {
+            Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (insert_accounting_structure). Exiting ..\n", config.name, config.type);
+            exit_plugin(1);
+          }
+        }
+      }
+
       elem_acc->packet_counter += data->pkt_num;
       elem_acc->flow_counter += data->flo_num;
       elem_acc->bytes_counter += data->pkt_len;
       elem_acc->tcp_flags = data->tcp_flags;
+      elem_acc->flow_type = data->flow_type;
       elem_acc->signature = hash; 
       if (config.what_to_count & COUNT_CLASS) {
         elem_acc->packet_counter += data->cst.pa;
@@ -277,5 +426,6 @@ void reset_counters(struct acc *elem)
   elem->packet_counter = 0;
   elem->bytes_counter = 0;
   elem->tcp_flags = 0;
+  elem->flow_type = 0;
   memcpy(&elem->rstamp, &cycle_stamp, sizeof(struct timeval));
 }
