@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2012 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2014 by Paolo Lucente
 */
 
 /*
@@ -32,10 +32,13 @@ EXT int cfg_key_syslog(char *, char *, char *);
 EXT int cfg_key_logfile(char *, char *, char *);
 EXT int cfg_key_pidfile(char *, char *, char *);
 EXT int cfg_key_daemonize(char *, char *, char *);
+EXT int cfg_key_proc_name(char *, char *, char *);
 EXT int cfg_key_aggregate(char *, char *, char *);
+EXT int cfg_key_aggregate_primitives(char *, char *, char *);
 EXT int cfg_key_snaplen(char *, char *, char *);
 EXT int cfg_key_aggregate_filter(char *, char *, char *);
 EXT int cfg_key_pcap_filter(char *, char *, char *);
+EXT int cfg_key_use_ip_next_hop(char *, char *, char *);
 EXT int cfg_key_interface(char *, char *, char *);
 EXT int cfg_key_interface_wait(char *, char *, char *);
 EXT int cfg_key_files_umask(char *, char *, char *);
@@ -63,6 +66,7 @@ EXT int cfg_key_sql_refresh_time(char *, char *, char *);
 EXT int cfg_key_sql_startup_delay(char *, char *, char *);
 EXT int cfg_key_sql_optimize_clauses(char *, char *, char *);
 EXT int cfg_key_sql_history(char *, char *, char *);
+EXT int cfg_key_sql_history_offset(char *, char *, char *);
 EXT int cfg_key_sql_history_roundoff(char *, char *, char *);
 EXT int cfg_key_sql_history_since_epoch(char *, char *, char *);
 EXT int cfg_key_sql_recovery_logfile(char *, char *, char *);
@@ -79,18 +83,26 @@ EXT int cfg_key_sql_aggressive_classification(char *, char *, char *);
 EXT int cfg_key_sql_locking_style(char *, char *, char *);
 EXT int cfg_key_sql_use_copy(char *, char *, char *);
 EXT int cfg_key_sql_delimiter(char *, char *, char *);
+EXT int cfg_key_timestamps_secs(char *, char *, char *);
+EXT int cfg_key_mongo_insert_batch(char *, char *, char *);
+EXT int cfg_key_amqp_exchange_type(char *, char *, char *);
+EXT int cfg_key_amqp_persistent_msg(char *, char *, char *);
+EXT int cfg_key_amqp_frame_max(char *, char *, char *);
 EXT int cfg_key_plugin_pipe_size(char *, char *, char *);
 EXT int cfg_key_plugin_pipe_backlog(char *, char *, char *);
 EXT int cfg_key_plugin_buffer_size(char *, char *, char *);
 EXT int cfg_key_networks_mask(char *, char *, char *);
 EXT int cfg_key_networks_file(char *, char *, char *);
+EXT int cfg_key_networks_file_filter(char *, char *, char *);
 EXT int cfg_key_networks_cache_entries(char *, char *, char *);
 EXT int cfg_key_ports_file(char *, char *, char *);
-EXT int cfg_key_refresh_maps(char *, char *, char *);
-EXT int cfg_key_print_refresh_time(char *, char *, char *);
 EXT int cfg_key_print_cache_entries(char *, char *, char *);
 EXT int cfg_key_print_markers(char *, char *, char *);
 EXT int cfg_key_print_output(char *, char *, char *);
+EXT int cfg_key_print_output_file(char *, char *, char *);
+EXT int cfg_key_print_output_file_append(char *, char *, char *);
+EXT int cfg_key_print_output_separator(char *, char *, char *);
+EXT int cfg_key_print_latest_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_port(char *, char *, char *);
 EXT int cfg_key_nfacctd_ip(char *, char *, char *);
 EXT int cfg_key_nfacctd_allow_file(char *, char *, char *);
@@ -100,21 +112,28 @@ EXT int cfg_key_nfacctd_as_new(char *, char *, char *);
 EXT int cfg_key_nfacctd_net(char *, char *, char *);
 EXT int cfg_key_nfacctd_disable_checks(char *, char *, char *);
 EXT int cfg_key_nfacctd_mcast_groups(char *, char *, char *);
-EXT int cfg_key_nfacctd_sql_log(char *, char *, char *);
+EXT int cfg_key_nfacctd_pipe_size(char *, char *, char *);
+EXT int cfg_key_nfacctd_pro_rating(char *, char *, char *);
+EXT int cfg_key_nfacctd_account_options(char *, char *, char *);
 EXT int cfg_key_pmacctd_force_frag_handling(char *, char *, char *);
 EXT int cfg_key_pmacctd_frag_buffer_size(char *, char *, char *);
 EXT int cfg_key_pmacctd_flow_buffer_size(char *, char *, char *);
 EXT int cfg_key_pmacctd_flow_buffer_buckets(char *, char *, char *);
 EXT int cfg_key_pmacctd_conntrack_buffer_size(char *, char *, char *);
 EXT int cfg_key_pmacctd_flow_lifetime(char *, char *, char *);
+EXT int cfg_key_pmacctd_flow_tcp_lifetime(char *, char *, char *);
 EXT int cfg_key_pmacctd_ext_sampling_rate(char *, char *, char *);
 EXT int cfg_key_sfacctd_renormalize(char *, char *, char *);
 EXT int cfg_key_pcap_savefile(char *, char *, char *);
+EXT int cfg_key_maps_refresh(char *, char *, char *);
+EXT int cfg_key_maps_index(char *, char *, char *);
+EXT int cfg_key_maps_entries(char *, char *, char *);
+EXT int cfg_key_maps_row_len(char *, char *, char *);
 EXT int cfg_key_pre_tag_map(char *, char *, char *);
-EXT int cfg_key_pre_tag_map_entries(char *, char *, char *);
 EXT int cfg_key_pre_tag_filter(char *, char *, char *);
 EXT int cfg_key_pre_tag2_filter(char *, char *, char *);
 EXT int cfg_key_post_tag(char *, char *, char *);
+EXT int cfg_key_post_tag2(char *, char *, char *);
 EXT int cfg_key_sampling_rate(char *, char *, char *);
 EXT int cfg_key_sampling_map(char *, char *, char *);
 EXT int cfg_key_classifiers(char *, char *, char *);
@@ -135,9 +154,22 @@ EXT int cfg_key_sfprobe_receiver(char *, char *, char *);
 EXT int cfg_key_sfprobe_agentip(char *, char *, char *);
 EXT int cfg_key_sfprobe_agentsubid(char *, char *, char *);
 EXT int cfg_key_sfprobe_ifspeed(char *, char *, char *);
+EXT int cfg_key_tee_receivers(char *, char *, char *);
 EXT int cfg_key_tee_transparent(char *, char *, char *);
+EXT int cfg_key_tee_max_receivers(char *, char *, char *);
+EXT int cfg_key_tee_max_receiver_pools(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp(char *, char *, char *);
-EXT int cfg_key_nfacctd_bgp_msglog(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_output(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_file(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_amqp_host(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_amqp_user(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_amqp_passwd(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_amqp_exchange(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_amqp_exchange_type(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_amqp_routing_key(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_amqp_persistent_msg(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_amqp_frame_max(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_amqp_retry(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_max_peers(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_ip(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_port(char *, char *, char *);
@@ -158,23 +190,44 @@ EXT int cfg_key_nfacctd_bgp_peer_src_as_map(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_src_local_pref_map(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_src_med_map(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_to_agent_map(char *, char *, char *);
-EXT int cfg_key_nfacctd_bgp_iface_to_rd_map(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_follow_default(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_follow_nexthop(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_neighbors_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_md5_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_table_peer_buckets(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_per_peer_buckets(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_attr_hash_buckets(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_per_peer_hash(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_output(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_file(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_refresh_time(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_amqp_host(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_amqp_user(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_amqp_passwd(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_amqp_exchange(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_amqp_exchange_type(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_amqp_routing_key(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_amqp_persistent_msg(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_amqp_frame_max(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_batch(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_batch_interval(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_pipe_size(char *, char *, char *);
+EXT int cfg_key_nfacctd_flow_to_rd_map(char *, char *, char *);
 EXT int cfg_key_nfacctd_isis(char *, char *, char *);
 EXT int cfg_key_nfacctd_isis_ip(char *, char *, char *);
 EXT int cfg_key_nfacctd_isis_net(char *, char *, char *);
 EXT int cfg_key_nfacctd_isis_iface(char *, char *, char *);
 EXT int cfg_key_nfacctd_isis_mtu(char *, char *, char *);
 EXT int cfg_key_nfacctd_isis_msglog(char *, char *, char *);
+EXT int cfg_key_igp_daemon_map(char *, char *, char *);
+EXT int cfg_key_igp_daemon_map_msglog(char *, char *, char *);
+EXT int cfg_key_geoip_ipv4_file(char *, char *, char *);
+EXT int cfg_key_geoip_ipv6_file(char *, char *, char *);
 EXT int cfg_key_uacctd_group(char *, char *, char *);
 EXT int cfg_key_uacctd_nl_size(char *, char *, char *);
 EXT int cfg_key_tunnel_0(char *, char *, char *);
-EXT int cfg_key_xlate_src(char *, char *, char *);
-EXT int cfg_key_xlate_dst(char *, char *, char *);
+EXT int cfg_key_pkt_len_distrib_bins(char *, char *, char *);
 
 EXT void parse_time(char *, char *, int *, int *);
+EXT void cfg_set_aggregate(char *, u_int64_t [], u_int64_t, char *);
 #undef EXT
