@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2012 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2014 by Paolo Lucente
 */
 
 /*
@@ -47,12 +47,13 @@ void pcap_cb(u_char *user, const struct pcap_pkthdr *pkthdr, const u_char *buf)
     pptrs.pkthdr = (struct pcap_pkthdr *) pkthdr;
     pptrs.packet_ptr = (u_char *) buf;
     pptrs.mac_ptr = 0; pptrs.vlan_ptr = 0; pptrs.mpls_ptr = 0;
+    pptrs.iph_ptr = 0; pptrs.tlh_ptr = 0; pptrs.payload_ptr = 0;
     pptrs.pf = 0; pptrs.shadow = 0; pptrs.tag = 0; pptrs.tag2 = 0;
-    pptrs.class = 0; pptrs.bpas = 0, pptrs.bta = 0; pptrs.blp = 0;
-    pptrs.bmed = 0; pptrs.bitr = 0;
+    pretag_free_label(&pptrs.label); pptrs.class = 0; pptrs.bpas = 0;
+    pptrs.bta = 0; pptrs.blp = 0; pptrs.bmed = 0; pptrs.bitr = 0;
+    pptrs.bta2 = 0; pptrs.bta_af = 0;
     pptrs.tun_layer = 0; pptrs.tun_stack = 0;
     pptrs.f_agent = cb_data->f_agent;
-    pptrs.idtable = cb_data->idt;
     pptrs.bpas_table = cb_data->bpas_table;
     pptrs.blp_table = cb_data->blp_table;
     pptrs.bmed_table = cb_data->bmed_table;
@@ -60,6 +61,7 @@ void pcap_cb(u_char *user, const struct pcap_pkthdr *pkthdr, const u_char *buf)
     pptrs.ifindex_in = cb_data->ifindex_in;
     pptrs.ifindex_out = cb_data->ifindex_out;
     pptrs.f_status = NULL;
+    pptrs.flow_type = NF9_FTYPE_TRAFFIC;
 
     (*device->data->handler)(pkthdr, &pptrs);
     if (pptrs.iph_ptr) {
@@ -68,21 +70,25 @@ void pcap_cb(u_char *user, const struct pcap_pkthdr *pkthdr, const u_char *buf)
           isis_srcdst_lookup(&pptrs);
         }
         if (config.nfacctd_bgp) {
-          PM_find_id((struct id_table *)pptrs.bta_table, &pptrs, &pptrs.bta, NULL);
+          BTA_find_id((struct id_table *)pptrs.bta_table, &pptrs, &pptrs.bta, &pptrs.bta2);
           bgp_srcdst_lookup(&pptrs);
         }
         if (config.nfacctd_bgp_peer_as_src_map) PM_find_id((struct id_table *)pptrs.bpas_table, &pptrs, &pptrs.bpas, NULL);
         if (config.nfacctd_bgp_src_local_pref_map) PM_find_id((struct id_table *)pptrs.blp_table, &pptrs, &pptrs.blp, NULL);
         if (config.nfacctd_bgp_src_med_map) PM_find_id((struct id_table *)pptrs.bmed_table, &pptrs, &pptrs.bmed, NULL);
-        if (config.pre_tag_map) PM_find_id((struct id_table *)pptrs.idtable, &pptrs, &pptrs.tag, &pptrs.tag2);
 
-        exec_plugins(&pptrs);
+	set_index_pkt_ptrs(&pptrs);
+        exec_plugins(&pptrs, &req);
       }
     }
   }
 
   if (reload_map) {
+    bta_map_caching = FALSE;
+    sampling_map_caching = FALSE;
+
     load_networks(config.networks_file, &nt, &nc);
+
     if (config.nfacctd_bgp && config.nfacctd_bgp_peer_as_src_map)
       load_id_file(MAP_BGP_PEER_AS_SRC, config.nfacctd_bgp_peer_as_src_map, (struct id_table *)cb_data->bpas_table, &req, &bpas_map_allocated);
     if (config.nfacctd_bgp && config.nfacctd_bgp_src_local_pref_map)
@@ -91,9 +97,9 @@ void pcap_cb(u_char *user, const struct pcap_pkthdr *pkthdr, const u_char *buf)
       load_id_file(MAP_BGP_SRC_MED, config.nfacctd_bgp_src_med_map, (struct id_table *)cb_data->bmed_table, &req, &bmed_map_allocated);
     if (config.nfacctd_bgp)
       load_id_file(MAP_BGP_TO_XFLOW_AGENT, config.nfacctd_bgp_to_agent_map, (struct id_table *)cb_data->bta_table, &req, &bta_map_allocated);
-    if (config.pre_tag_map)
-      load_id_file(config.acct_type, config.pre_tag_map, (struct id_table *) pptrs.idtable, &req, &tag_map_allocated);
+
     reload_map = FALSE;
+    gettimeofday(&reload_map_tstamp, NULL);
   }
 }
 
@@ -121,7 +127,8 @@ int ip_handler(register struct packet_ptrs *pptrs)
   if (config.handle_fragments) {
     if (pptrs->l4_proto == IPPROTO_TCP || pptrs->l4_proto == IPPROTO_UDP) {
       if (off+MyTLHdrSz > caplen) {
-        Log(LOG_INFO, "INFO ( default/core ): short IPv4 packet read (%u/%u/frags). Snaplen issue ?\n", caplen, off+MyTLHdrSz);
+        Log(LOG_INFO, "INFO ( %s/core ): short IPv4 packet read (%u/%u/frags). Snaplen issue ?\n",
+			config.name, caplen, off+MyTLHdrSz);
         return FALSE;
       }
       pptrs->tlh_ptr = ptr;
@@ -165,7 +172,8 @@ int ip_handler(register struct packet_ptrs *pptrs)
 
       if (pptrs->l4_proto == IPPROTO_TCP) {
         if (off_l4+TCPFlagOff+1 > caplen) {
-          Log(LOG_INFO, "INFO ( default/core ): short IPv4 packet read (%u/%u/flows). Snaplen issue ?\n", caplen, off_l4+TCPFlagOff+1);
+          Log(LOG_INFO, "INFO ( %s/core ): short IPv4 packet read (%u/%u/flows). Snaplen issue ?\n",
+			config.name, caplen, off_l4+TCPFlagOff+1);
           return FALSE;
         }
         if (((struct my_tcphdr *)pptrs->tlh_ptr)->th_flags & TH_SYN) pptrs->tcp_flags |= TH_SYN;
@@ -221,7 +229,7 @@ int ip6_handler(register struct packet_ptrs *pptrs)
   /* length checks */
   if (off+IP6HdrSz > caplen) return FALSE; /* IP packet truncated */
   if (plen == 0 && ((struct ip6_hdr *)pptrs->iph_ptr)->ip6_nxt != IPPROTO_NONE) {
-    Log(LOG_INFO, "INFO ( default/core ): NULL IPv6 payload length. Jumbo packets are currently not supported.\n");
+    Log(LOG_INFO, "INFO ( %s/core ): NULL IPv6 payload length. Jumbo packets are currently not supported.\n", config.name);
     return FALSE;
   }
 
@@ -266,7 +274,8 @@ int ip6_handler(register struct packet_ptrs *pptrs)
   if (config.handle_fragments) {
     if (pptrs->l4_proto == IPPROTO_TCP || pptrs->l4_proto == IPPROTO_UDP) {
       if (off+MyTLHdrSz > caplen) {
-        Log(LOG_INFO, "INFO ( default/core ): short IPv6 packet read (%u/%u/frags). Snaplen issue ?\n", caplen, off+MyTLHdrSz);
+        Log(LOG_INFO, "INFO ( %s/core ): short IPv6 packet read (%u/%u/frags). Snaplen issue ?\n",
+			config.name, caplen, off+MyTLHdrSz);
         return FALSE;
       }
 
@@ -308,7 +317,8 @@ int ip6_handler(register struct packet_ptrs *pptrs)
 
       if (pptrs->l4_proto == IPPROTO_TCP) {
         if (off_l4+TCPFlagOff+1 > caplen) {
-          Log(LOG_INFO, "INFO ( default/core ): short IPv6 packet read (%u/%u/flows). Snaplen issue ?\n", caplen, off_l4+TCPFlagOff+1);
+          Log(LOG_INFO, "INFO ( %s/core ): short IPv6 packet read (%u/%u/flows). Snaplen issue ?\n",
+			config.name, caplen, off_l4+TCPFlagOff+1);
           return FALSE;
         }
         if (((struct my_tcphdr *)pptrs->tlh_ptr)->th_flags & TH_SYN) pptrs->tcp_flags |= TH_SYN;
@@ -331,43 +341,50 @@ int ip6_handler(register struct packet_ptrs *pptrs)
 }
 #endif
 
-void PM_find_id(struct id_table *t, struct packet_ptrs *pptrs, pm_id_t *tag, pm_id_t *tag2)
+int PM_find_id(struct id_table *t, struct packet_ptrs *pptrs, pm_id_t *tag, pm_id_t *tag2)
 {
-  int x, j, stop;
-  pm_id_t id;
+  int x, j;
+  pm_id_t ret = 0;
 
-  if (!t) return;
+  if (!t) return 0;
 
-  id = 0;
+  pretag_init_vars(pptrs, t);
   if (tag) *tag = 0;
   if (tag2) *tag2 = 0;
-  for (x = 0; x < t->ipv4_num; x++) {
-    t->e[x].last_matched = FALSE;
-    for (j = 0, stop = 0; !stop; j++) stop = (*t->e[x].func[j])(pptrs, &id, &t->e[x]);
-    if (id) {
-      if (stop == PRETAG_MAP_RCODE_ID) {
-        if (t->e[x].stack.func) id = (*t->e[x].stack.func)(id, *tag);
-        *tag = id;
-      }
-      else if (stop == PRETAG_MAP_RCODE_ID2) {
-        if (t->e[x].stack.func) id = (*t->e[x].stack.func)(id, *tag2);
-        *tag2 = id;
-      }
+  if (pptrs) {
+    pptrs->have_tag = FALSE;
+    pptrs->have_tag2 = FALSE;
+  }
 
-      if (t->e[x].jeq.ptr) {
-        if (t->e[x].ret) {
-          exec_plugins(pptrs);
-          set_shadow_status(pptrs);
-          *tag = 0;
-          *tag2 = 0;
-        }
+  /* Giving a first try with index(es) */
+  if (config.maps_index && pretag_index_have_one(t)) {
+    struct id_entry *index_results[ID_TABLE_INDEX_RESULTS];
+    u_int32_t iterator;
+
+    pretag_index_lookup(t, pptrs, index_results, ID_TABLE_INDEX_RESULTS);
+
+    for (iterator = 0; index_results[iterator] && iterator < ID_TABLE_INDEX_RESULTS; iterator++) {
+      ret = pretag_entry_process(index_results[iterator], pptrs, tag, tag2);
+      if (!(ret & PRETAG_MAP_RCODE_JEQ)) return ret;
+    }
+
+    /* if we have at least one index we trust we did a good job */
+    return ret;
+  }
+
+  for (x = 0; x < t->ipv4_num; x++) {
+    ret = pretag_entry_process(&t->e[x], pptrs, tag, tag2);
+
+    if (!ret || ret > TRUE) {
+      if (ret & PRETAG_MAP_RCODE_JEQ) {
         x = t->e[x].jeq.ptr->pos;
-        x--; /* yes, it will be automagically incremented by the for() cycle */
-        id = 0;
+        x--; // yes, it will be automagically incremented by the for() cycle
       }
       else break;
     }
   }
+
+  return ret;
 }
 
 void compute_once()
@@ -379,6 +396,11 @@ void compute_once()
   PpayloadSz = sizeof(struct pkt_payload);
   PextrasSz = sizeof(struct pkt_extras);
   PbgpSz = sizeof(struct pkt_bgp_primitives);
+  PnatSz = sizeof(struct pkt_nat_primitives);
+  PmplsSz = sizeof(struct pkt_mpls_primitives);
+  PvhdrSz = sizeof(struct pkt_vlen_hdr_primitives);
+  PmLabelTSz = sizeof(pm_label_t);
+  PtLabelTSz = sizeof(pt_label_t);
   ChBufHdrSz = sizeof(struct ch_buf_hdr);
   CharPtrSz = sizeof(char *);
   IP4HdrSz = sizeof(struct my_iphdr);
@@ -429,7 +451,7 @@ int gtp_tunnel_configurator(struct tunnel_handler *th, char *opts)
   }
   else {
     th->tf = NULL;
-    Log(LOG_WARNING, "WARN ( default/core ): GTP tunnel handler not loaded due to invalid options: '%s'\n", opts);
+    Log(LOG_WARNING, "WARN ( %s/core ): GTP tunnel handler not loaded due to invalid options: '%s'\n", config.name, opts);
   }
 
   return 0;
@@ -438,17 +460,35 @@ int gtp_tunnel_configurator(struct tunnel_handler *th, char *opts)
 int gtp_tunnel_func(register struct packet_ptrs *pptrs)
 {
   register u_int16_t caplen = ((struct pcap_pkthdr *)pptrs->pkthdr)->caplen;
-  struct my_gtphdr *gtp_hdr = (struct my_gtphdr *) pptrs->payload_ptr;
+  struct my_gtphdr_v0 *gtp_hdr_v0 = (struct my_gtphdr_v0 *) pptrs->payload_ptr;
+  struct my_gtphdr_v1 *gtp_hdr_v1 = (struct my_gtphdr_v1 *) pptrs->payload_ptr;
   struct my_udphdr *udp_hdr = (struct my_udphdr *) pptrs->tlh_ptr;
-  u_int16_t off = pptrs->payload_ptr-pptrs->packet_ptr, gtp_len;
+  u_int16_t off = pptrs->payload_ptr-pptrs->packet_ptr;
+  u_int16_t gtp_hdr_len, gtp_opt_len, gtp_version;
   char *ptr = pptrs->payload_ptr;
-  int ret;
+  int ret, trial;
 
-  if (off+sizeof(struct my_gtphdr) < caplen) {
-    gtp_len = (ntohs(udp_hdr->uh_ulen)-sizeof(struct my_udphdr))-ntohs(gtp_hdr->length);
-    if (off+gtp_len < caplen) {
-      off += gtp_len;
-      ptr += gtp_len;
+  gtp_version = (gtp_hdr_v0->flags >> 5) & 0x07;
+
+  switch (gtp_version) {
+  case 0:
+    gtp_hdr_len = 4;
+    break;
+  case 1:
+    gtp_hdr_len = 8;
+    break;
+  default:
+    Log(LOG_INFO, "INFO ( %s/core ): unsupported GTP version %u\n", config.name, gtp_version);
+    return FALSE;
+  }
+
+  if (off + gtp_hdr_len < caplen) {
+    off += gtp_hdr_len;
+    ptr += gtp_hdr_len;
+    ret = 0; trial = 0;
+
+    while (!ret && trial < MAX_GTP_TRIALS) {
+      off++; ptr++; trial++;
 
       pptrs->iph_ptr = ptr;
       pptrs->tlh_ptr = NULL; pptrs->payload_ptr = NULL;
@@ -497,15 +537,26 @@ int gtp_tunnel_func(register struct packet_ptrs *pptrs)
 	break;
       }
     }
-    else {
-      Log(LOG_INFO, "INFO ( default/core ): short GTP packet read (%u/%u/tunnel/#1). Snaplen issue ?\n", caplen, off+gtp_len);
-      return FALSE;
-    }
-  } 
+  }
   else {
-    Log(LOG_INFO, "INFO ( default/core ): short GTP packet read (%u/%u/tunnel/#2). Snaplen issue ?\n", caplen, off+sizeof(struct my_gtphdr));
+    Log(LOG_INFO, "INFO ( %s/core ): short GTP packet read (%u/%u/tunnel). Snaplen issue ?\n",
+			config.name, caplen, off + gtp_hdr_len);
     return FALSE;
   }
 
   return ret;
 }
+
+void set_index_pkt_ptrs(struct packet_ptrs *pptrs)
+{
+  pptrs->pkt_data_ptrs[CUSTOM_PRIMITIVE_PACKET_PTR] = pptrs->packet_ptr;
+  pptrs->pkt_data_ptrs[CUSTOM_PRIMITIVE_MAC_PTR] = pptrs->mac_ptr;
+  pptrs->pkt_data_ptrs[CUSTOM_PRIMITIVE_VLAN_PTR] = pptrs->vlan_ptr;
+  pptrs->pkt_data_ptrs[CUSTOM_PRIMITIVE_MPLS_PTR] = pptrs->mpls_ptr;
+  pptrs->pkt_data_ptrs[CUSTOM_PRIMITIVE_L3_PTR] = pptrs->iph_ptr;
+  pptrs->pkt_data_ptrs[CUSTOM_PRIMITIVE_L4_PTR] = pptrs->tlh_ptr;
+  pptrs->pkt_data_ptrs[CUSTOM_PRIMITIVE_PAYLOAD_PTR] = pptrs->payload_ptr;
+
+  pptrs->pkt_proto[CUSTOM_PRIMITIVE_L3_PTR] = pptrs->l3_proto;
+  pptrs->pkt_proto[CUSTOM_PRIMITIVE_L4_PTR] = pptrs->l4_proto;
+}
