@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2012 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2013 by Paolo Lucente
 */
 
 /*
@@ -42,6 +42,9 @@ EXT void dst_mac_handler(struct channels_list_entry *, struct packet_ptrs *, cha
 EXT void vlan_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void cos_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void etype_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void mpls_label_top_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void mpls_label_bottom_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void mpls_stack_depth_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void src_host_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void dst_host_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void src_port_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -51,14 +54,19 @@ EXT void ip_proto_handler(struct channels_list_entry *, struct packet_ptrs *, ch
 EXT void tcp_flags_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void counters_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void counters_renormalize_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-EXT void id_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void post_tag_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void post_tag2_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void flows_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void class_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void in_iface_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void out_iface_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void sampling_rate_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void timestamp_start_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void custom_primitives_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void sfprobe_payload_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void tee_payload_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void nfprobe_extras_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void mpls_vpn_rd_frommap_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 
 EXT void NF_src_mac_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void NF_dst_mac_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -87,10 +95,24 @@ EXT void NF_flows_handler(struct channels_list_entry *, struct packet_ptrs *, ch
 EXT void NF_class_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void NF_in_iface_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void NF_out_iface_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-EXT void NF_id_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-EXT void NF_id2_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_sampling_rate_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_post_nat_src_host_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_post_nat_dst_host_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_post_nat_src_port_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_post_nat_dst_port_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_nat_event_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_mpls_label_top_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_mpls_label_bottom_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_mpls_stack_depth_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_mpls_vpn_rd_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_timestamp_start_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_timestamp_end_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_custom_primitives_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void NF_counters_renormalize_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void NF_counters_map_renormalize_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_cust_tag_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_cust_tag2_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void NF_cust_label_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 
 EXT void bgp_ext_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void nfprobe_bgp_ext_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -98,14 +120,10 @@ EXT void sfprobe_bgp_ext_handler(struct channels_list_entry *, struct packet_ptr
 EXT void bgp_peer_src_as_frommap_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void bgp_src_local_pref_frommap_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void bgp_src_med_frommap_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-EXT void bgp_src_net_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-EXT void bgp_dst_net_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void bgp_src_nmask_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void bgp_dst_nmask_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void bgp_peer_dst_ip_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 
-EXT void igp_src_net_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-EXT void igp_dst_net_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void igp_src_nmask_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void igp_dst_nmask_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void igp_peer_dst_ip_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -131,12 +149,14 @@ EXT void SF_tcp_flags_handler(struct channels_list_entry *, struct packet_ptrs *
 EXT void SF_counters_new_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_counters_renormalize_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_counters_map_renormalize_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-EXT void SF_id_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-EXT void SF_id2_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void SF_tag_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void SF_tag2_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_class_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_in_iface_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_out_iface_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void SF_sampling_rate_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_sampling_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void SF_timestamp_start_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_bgp_peer_src_as_fromext_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_bgp_peer_src_as_fromstd_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_as_path_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
@@ -145,10 +165,18 @@ EXT void SF_peer_dst_as_handler(struct channels_list_entry *, struct packet_ptrs
 EXT void SF_local_pref_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void SF_std_comms_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 
-EXT void ptag_id_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
-EXT void ptag_id2_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void pre_tag_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void pre_tag2_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void pre_tag_label_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void sampling_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 EXT void sfprobe_sampling_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
 
+#if defined (WITH_GEOIP)
+EXT void geoip_init();
+EXT void src_host_country_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+EXT void dst_host_country_handler(struct channels_list_entry *, struct packet_ptrs *, char **);
+#endif
+
 EXT int evaluate_lm_method(struct packet_ptrs *, u_int8_t, u_int32_t, u_int32_t);
+EXT char *lookup_tpl_ext_db(void *, u_int32_t, u_int16_t);
 #undef EXT
